module ade_comf; 


{============================================================================}
{===  mpi_ari  ======   Analytic Differentiable Expression Arithmetic =======}
{====================   =        =              =          ===        =======}
{============================================================================}
{                                                                            }
{    File    :  inte_ari.p                                                     }
{                                                                            }
{=   Version :  1.1                                                          }
{                                                                            }
{=   Date    :  05/../..  ( first implementation )                           }
{=              ../../..  ( last change )                                    }
{                                                                            }
{    Language:  PASCAL-XSC, Vers. 3.0 or later                               }
{                                                                            }
{    Author  :  K. Petras                                                    }
{                                                                            }
{============================================================================}
{      Das Modul behandelt komplexe Rechtecke um Intervalle                  }
{      Es  rechnet gleichzeitig mit Intervallen und solchen                  }
{      zur reellen Achse symmetrischen Rechtecken und prueft                 }
{      jeweils, ob die Funktion dort analytisch ist.                         }
{============================================================================}
   
 use global i_ari, arith; 


global type inte_type = global record
rint, dint : interval;
cint	   : icom;
   a, d	   : boolean;
end;	   

global var type_switch:integer;



global var rerror : boolean; 
   
global function iconstr(x	 : interval;
			  r1, r2 : real) : inte_type;
var h : interval;
begin
   h:= (sup(x)-inf(x))/2; 
   iconstr.rint := x;
   iconstr.dint := 1;
   iconstr.cint := icompl(x+h*intval(-abs(r1-1), abs(r1-1)),
			    sup(h)*abs(r2));
   iconstr.a    := true;
   iconstr.d    := true;
end; { iconstr }
  
global function iconstr(x : interval) : inte_type;
begin
   iconstr.rint := x;
   iconstr.a := true;
end; { iconstr }

global function iconstr(x    : interval;
			  r    : real) : inte_type;
var s :real ; 
begin
   case type_switch of	 
      1: iconstr.rint:= x;
      2: iconstr:= iconstr(x);
      3: begin		 
	     s:= abs(r); 
	     iconstr := iconstr(x, (s+>1/>s)/2, (s->1/>s)/2);
	 end;		 
   end;			 
end; { iconstr }	 



global procedure write(var f : text;
		    x : inte_type);
begin
   case type_switch of 
      1:   write(f,x.rint);
      2: write(f,x.rint);
      3: begin
	    writeln(f,'reell_intv = [', inf(x.rint):20,
		    ' , ',sup(x.rint):20,' ]	');
	    if (x.d) then
	       writeln(f, 'abl_intv   = [', inf(x.dint):20,
		       ' , ',sup(x.dint):20,' ]')
	    else
	       writeln(f, 'keine Ableitungseinschliessung berechnet'); 
	    if (x.a) then
	    begin
	       writeln(f, 'compl_intv =   [', inf(x.cint.re):20,
		       ' , ',sup(x.cint.re):20,' ]');
	       writeln(f, '    + i   [', -x.cint.im:20,
		       ' , ',x.cint.im:20,' ]');
	    end
	    else
	       writeln(f, 'keine komplexe Einschliessung berechnet'); 
	 end;
   end;
end;



global operator := (var n : integer;
		    b : boolean ); 
begin
   if (b) then
      n:=1
   else
      n:=0;
end;

global function max(x,y :real ): real; 
begin
   if (x>y) then
      max:=x
   else
      max:=y;
end;

global function min(x,y :integer):integer; 
begin
   if (x>y) then
      min:=y
   else
      min:=x;
end;

   
{==========   Hilfsfunktionen ============}

global function dia (x	: icom):real;
begin
   if ((abs(x.re.sup) <1.0e150) and 
       (abs(x.re.inf) <1.0e150) and 
       (x.im <1.0e150) ) then
       dia := 1.00000001*sqrt(diam(x.re)+ sqr(2*x.im))
   else
      dia := diam(x.re) + 2*x.im 
end;

global function max (n, m : integer):integer;
begin
   if (n>m) then
      max:=n
   else
      max:=m;
end; { max }


{==========   Vorzeichenumkehr =================================}

global operator -  ( x : inte_type) ch : inte_type;
begin
   case type_switch of
      1: ch.rint:= -x.rint;
      2: begin
	    ch.rint:= -x.rint;
	    ch.a:=x.a;
	 end;
      3: begin	 
	    ch.rint:=-x.rint;
	    ch.dint:=-x.dint;
	    ch.cint.re:=-x.cint.re;
	    ch.cint.re:=x.cint.im;
	    ch.a:=x.a;
	    ch.d:=x.d;
	 end;
   end;
   end;
      
{==========   Addition mit analytischer Beteiligung ============}

global operator +  ( x,y : inte_type) add : inte_type;
begin
   case type_switch of	
      1:  add.rint:= x.rint+y.rint;
      2: begin
	    if (x.a and y.a) then
	    begin
	       add.rint := x.rint + y.rint; 
	       add.a    := true;
	    end
	    else
	    begin
	       add.rint := 0; 
	       add.a   := false;
	    end;
	 end;
      3: begin
	    if (rerror) then
	       add :=iconstr(intval(0), 0.0, 0.0)
	    else
	    begin
	       if (x.d and y.d) then
	       begin
		  add.dint :=x.dint + y.dint; 
		  add.d    := true;
	       end
	       else
	       begin
		  add.dint :=0; 
		  add.d    := false;
	       end;
	       if (x.a and y.a) then
	       begin
		  add.rint := x.rint + y.rint; 
		  add.cint := x.cint + y.cint; 
		  add.a    := true;
	       end
	       else
	       begin
		  add.rint := x.rint + y.rint; 
		  add.cint := icompl(intval(0)); 
		  add.a    := false;
	       end;
	    end;
	    
	 end;
   end;
end;      





global operator +  (x : inte_type;
		    y : interval ) add : inte_type;
begin
   case type_switch of
      1: add.rint := x.rint+y;
      2: add := x+iconstr(y); 
      3: if (rerror) then
	    add :=iconstr(intval(0), 0.0, 0.0)
	 else
	 begin
	    if (x.d) then
	    begin
	       add.dint :=x.dint ; 
	       add.d    := true;
	    end
	    else
	    begin
	       add.dint :=0; 
	       add.d    := false;
	    end;
	    if (x.a) then
	    begin
	       add.rint := x.rint + y; 
	       add.cint := x.cint + y; 
	       add.a    := true;
	    end
	    else
	    begin
	       add.rint := x.rint + y; 
	       add.cint := icompl(intval(0)); 
	       add.a    := false;
	    end;
	 end;
   end;
end; { add }

global operator +  (x : interval;
		    y : inte_type ) add : inte_type;
begin
   case type_switch of
      1: add.rint := x+y.rint;
      2: add := y+iconstr(x); 
      3: if (rerror) then
	    add :=iconstr(intval(0), 0.0, 0.0)
	 else
	 begin
	    if (y.d) then
	    begin
	       add.dint :=y.dint ; 
	       add.d    := true;
	    end
	    else
	    begin
	       add.dint :=0; 
	       add.d    := false;
	    end;
	    if (y.a) then
	    begin
	       add.rint := y.rint + x; 
	       add.cint := y.cint + x; 
	       add.a    := true;
	    end
	    else
	    begin
	       add.rint := y.rint + x; 
	       add.cint := icompl(intval(0)); 
	       add.a    := false;
	    end;
	 end;
   end;
end; { add }

global operator +  (x : real;
		    y : inte_type ) add : inte_type;
begin
   add:=intval(x)+y;
end; { ade_neu }

global operator +  (x : integer;
		    y : inte_type ) add : inte_type;
begin
   add:=intval(x)+y;
end;

global operator +  ( y : inte_type; x : real) add : inte_type;
begin
   add:=intval(x)+y;
end; { ade_neu }

global operator +  ( y : inte_type; x : integer) add : inte_type;
begin
   add:=intval(x)+y;
end; { ade_neu }



{==========   Subtraktion mit analytischer Beteiligung ============}

global operator -  ( x,y : inte_type) sub : inte_type;
begin
   case type_switch of	
      1:  sub.rint:= x.rint-y.rint;
      2: begin
	    if (x.a and y.a) then
	    begin
	       sub.rint := x.rint - y.rint; 
	       sub.a    := true;
	    end
	    else
	    begin
	       sub.rint := 0; 
	       sub.a   := false;
	    end;
	 end;
      3: begin
	 if (rerror) then
	    sub :=iconstr(intval(0), 0.0, 0.0)
	 else
	 begin
	    if (x.d and y.d) then
	    begin
	       sub.dint :=x.dint - y.dint; 
	       sub.d    := true;
	    end
	    else
	    begin
	       sub.dint :=0; 
	       sub.d    := false;
	    end;
	    if (x.a and y.a) then
	    begin
	       sub.rint := x.rint - y.rint; 
	       sub.cint := x.cint - y.cint; 
	       sub.a    := true;
	    end
	    else
	    begin
	       sub.rint := x.rint - y.rint; 
	       sub.cint := icompl(intval(0)); 
	       sub.a    := false;
	    end;
	 end;
	 
      end;
   end;
end;      





global operator -  (x : inte_type;
		    y : interval ) sub : inte_type;
begin
   case type_switch of
      1: sub.rint := x.rint-y;
      2: sub := x-iconstr(y); 
      3: if (rerror) then
	    sub :=iconstr(intval(0), 0.0, 0.0)
	 else
	 begin
	    if (x.d) then
	    begin
	       sub.dint :=x.dint ; 
	       sub.d    := true;
	    end
	    else
	    begin
	       sub.dint :=0; 
	       sub.d    := false;
	    end;
	    if (x.a) then
	    begin
	       sub.rint := x.rint - y; 
	       sub.cint := x.cint - y; 
	       sub.a    := true;
	    end
	    else
	    begin
	       sub.rint := x.rint - y; 
	       sub.cint := icompl(intval(0)); 
	       sub.a    := false;
	    end;
	 end;
   end;
end; { sub }

global operator -  (x : interval;
		    y : inte_type ) sub : inte_type;
begin
   case type_switch of
      1: sub.rint := x-y.rint;
      2: sub := iconstr(x)-y;
      3: if (rerror) then
	    sub :=iconstr(intval(0), 0.0, 0.0)
	 else
	 begin
	    if (y.d) then
	    begin
	       sub.dint :=-y.dint ; 
	       sub.d    := true;
	    end
	    else
	    begin
	       sub.dint :=0; 
	       sub.d    := false;
	    end;
	    if (y.a) then
	    begin
	       sub.rint := x-y.rint ; 
	       sub.cint := x-y.cint; 
	       sub.a    := true;
	    end
	    else
	    begin
	       sub.rint := x-y.rint; 
	       sub.cint := icompl(intval(0)); 
	       sub.a    := false;
	    end;
	 end;
   end;
end; { sub }

      
global operator -  (x : real;
		    y : inte_type ) sub : inte_type;
begin
   sub:=intval(x)-y;
end; { ade_neu }

operator -  (x : integer;  y : inte_type ) sub : inte_type;
begin
   sub:=intval(x)-y;
end;

global operator -  ( y : inte_type; x : real) sub : inte_type;
begin
   sub:=y-intval(x);
end; { ade_neu }

global operator -  ( y : inte_type; x : integer) sub : inte_type;
begin
   sub:=y-intval(x);
end; { ade_neu }



{==========   Multiplikation mit analytischer Beteiligung ============}

global operator *  ( x,y : inte_type) prod : inte_type;
begin
   case type_switch of
      1:
	prod.rint:= x.rint*y.rint; 
      2: 
	 if (x.a and y.a) then
	 begin
	    prod.rint := x.rint * y.rint; 
	    prod.a    := true;
	 end
	 else
	 begin
	    prod.rint := 0; 
	    prod.a   := false;
	 end;
      3:
	if (rerror) then
	   prod :=iconstr(intval(0), 0.0, 0.0)
	else
	begin
	   if (x.d and y.d) then
	   begin
	      prod.dint := x.dint*y.rint + x.rint*y.dint; 
	      prod.d    := true;
	   end
	   else
	   begin
	      prod.dint :=0; 
	      prod.d    := false;
	   end;
	   if (x.a and y.a) then
	   begin
	      if (max(expo(sup(abs(x.cint.re))), 0)
		  + max(expo(x.cint.im), 0)
		  + max(expo(sup(abs(y.cint.re))), 0)
		  +  max(expo(y.cint.im), 0) <1020) then
	      begin
		 prod.rint := x.rint * y.rint; 
		 prod.cint := x.cint*y.cint; 
		 prod.a    := true;
	      end
	      else
	      begin
		 prod.rint := x.rint * y.rint; 
		 prod.cint := icompl(intval(0)); 
		 prod.a    := false;
	      end
	   end
	   else
	   begin
	      prod.rint := x.rint * y.rint; 
	      prod.cint := icompl(intval(0)); 
	      prod.a    := false;
	   end;
	end;
   end; 
end;
 

global operator *  (x : inte_type;
		    y : interval ) prod : inte_type;
begin
   case type_switch of
      1:
	prod.rint:= x.rint*y; 
      2: 
	 if (x.a) then
	 begin
	    prod.rint := x.rint * y; 
	    prod.a    := true;
	 end
	 else
	 begin
	    prod.rint := 0; 
	    prod.a   := false;
	 end;
      3:
	if (rerror) then
	   prod :=iconstr(intval(0), 0.0, 0.0)
	else
	begin
	   if (x.d) then
	   begin
	      prod.dint := x.dint*y     ; 
	      prod.d    := true;
	   end
	   else
	   begin
	      prod.dint :=0; 
	      prod.d    := false;
	   end;
	   if (x.a) then
	   begin
	      prod.rint := x.rint * y; 
	      prod.cint := x.cint*y; 
	      prod.a    := true;
	   end
	   else
	   begin
	      prod.rint := x.rint * y; 
	      prod.cint := icompl(intval(0)); 
	      prod.a    := false;
	   end;
	end;
   end; { prod }
   end;


global operator *  (x : interval;
		    y : inte_type ) prod : inte_type;
begin
   case type_switch of
      1:
	prod.rint:= x*y.rint; 
      2: 
	 if (y.a) then
	 begin
	    prod.rint := x * y.rint; 
	    prod.a    := true;
	 end
	 else
	 begin
	    prod.rint := 0; 
	    prod.a   := false;
	 end;
      3:
	if (rerror) then
	   prod :=iconstr(intval(0), 0.0, 0.0)
	else
	begin
	   if (y.d) then
	   begin
	      prod.dint := x *y.dint; 
	      prod.d    := true;
	   end
	   else
	   begin
	      prod.dint :=0; 
	      prod.d    := false;
	   end;
	   if (y.a) then
	   begin
	      prod.rint := x * y.rint; 
	      prod.cint :=  y.cint * x;  
	      prod.a    := true;
	   end
	   else
	   begin
	      prod.rint := x * y.rint; 
	      prod.cint := icompl(intval(0)); 
	      prod.a    := false;
	   end;
	end;
end; { prod }
   end;

global operator *  (x : real;
		    y : inte_type ) mul : inte_type;
begin
   mul:=intval(x)*y;
end; { ade_neu }

 operator *  (x : integer;
		    y : inte_type ) mul : inte_type;
begin
   mul:=intval(x)*y;
end;

global operator *  ( y : inte_type; x : real) mul : inte_type;
begin
   mul:=y*intval(x);
end; { ade_neu }

global operator *  ( y : inte_type; x : integer) mul : inte_type;
begin
   mul:=y*intval(x);
end; { ade_neu }


{==========   Division mit analytischer Beteiligung ============}

global operator /  ( x,y : inte_type) quot : inte_type;
var xdy : interval; 
begin
   case type_switch of
      1:
	quot.rint:= x.rint/y.rint; 
      2: 
	   if (0 in y.rint) then
	   begin
	      quot.rint := 0;
	      quot.a   := false;
	   end
	   else
	      if (x.a and y.a) then
	      begin
		 quot.rint := x.rint / y.rint; 
		 quot.a    := true;
	      end
	      else
	      begin
		 quot.rint := 0; 
		 quot.a   := false;
	      end;

      3:
	if (rerror) then
	   quot :=iconstr(intval(0), 0.0, 0.0)
	else
	begin
	   if (0 in y.rint) then
	   begin
	      quot := iconstr(intval(0.0), 0.0, 0.0);
	      rerror := true;
	   end
	   else
	   begin
	      if (x.a and y.a) then
	      begin
		 if (0 in y.cint.re) then
	 begin 
	    xdy := x.rint / y.rint; 
	    quot.cint := icompl(intval(0)); 
	    quot.a    := false;
	 end
	 else
	 begin
	    quot.cint :=x.cint/y.cint;
	    quot.a    := true;
	 end;
	      end
	      else
	      begin 
		 xdy := x.rint / y.rint; 
		 quot.cint := icompl(intval(0)); 
		 quot.a    := false;
	      end;
	      quot.rint := xdy; 
	      if (x.d and y.d) then
	      begin
		 quot.dint := (x.dint-y.dint*xdy)/y.rint; 
		 quot.d    := true;
	      end
	      else
	      begin
		 quot.dint :=0; 
		 quot.d    := false;
	      end;
	   end;      
	end;
      end;
   end; { quot }

global operator /  (x : inte_type;
		    y : interval ) quot : inte_type;
var edy	: interval;
begin
   case type_switch of
      1:
	quot.rint:= x.rint/y; 
      2: 
	  quot := x/iconstr(y); 

      3:
	if (rerror) then
	   quot :=iconstr(intval(0), 0.0, 0.0)
	else
	begin
	   if (0 in y) then
	   begin
	      quot := iconstr(intval(0.0), 0.0, 0.0);
	      rerror := true;
	   end
	   else
	   begin
	      edy:=1/y;
	      if (x.d) then
	      begin
		 quot.dint := edy*x.dint; 
		 quot.d    := true;
	      end
	      else
	      begin
		 quot.dint :=0; 
		 quot.d    := false;
	      end;
	      if (x.a) then
	      begin
		 quot.rint := edy*x.rint ; 
		 quot.cint :=icompl(edy*x.cint.re,  sup(abs(edy))*>x.cint.im); 
		 quot.a    := true;
	      end
	      else
	      begin 
		 quot.rint := x.rint / y; 
		 quot.cint := icompl(intval(0)); 
		 quot.a    := false;
	      end;
	   end;
	end;
end; { quot }
   end;

global operator /  (x : interval;
		    y : inte_type) quot : inte_type;
var c, faktor : interval;
   d	      : real;
begin
   case type_switch of
      1:
	quot.rint:= x/y.rint; 
      2: 
	  quot := iconstr(x)/y; 

      3:
	if (rerror) then
	   quot :=iconstr(intval(0), 0.0, 0.0)
	else
	begin
	   if (0 in y.rint) then
	   begin
	      quot := iconstr(intval(0.0), 0.0, 0.0);
	      rerror := true;
	   end
	   else
	   begin
	      if (y.d) then
	      begin
		 quot.dint := -x*y.dint/sqr(y.rint); 
		 quot.d    := true;
	      end
	      else
	      begin
		 quot.dint :=0; 
		 quot.d    := false;
	      end;
	      if ( y.a) then
	      begin
		 if (0 in y.cint.re) then
		 begin 
		    quot.rint := x / y.rint; 
		    quot.cint := icompl(intval(0)); 
		    quot.a    := false;
		 end
		 else
		 begin
		    quot.rint := x / y.rint; 
		    c:=re(y.cint);
		    d:=im(y.cint);
		    faktor:=x/(sqr(c)+intval(0,d*>d));
		    quot.cint := icompl(faktor*c, sup(abs(faktor))*>d); 
		    quot.a    := true;
		 end;
	      end
	      else
	 begin 
	    quot.rint := x / y.rint; 
	    quot.cint := icompl(intval(0)); 
	    quot.a    := false;
	 end;
	   end;     
	end; 
   end; { quot }
   end;
    

global operator /  (x : real;
		    y : inte_type ) quot : inte_type;
begin
   quot:=intval(x)/y;
end; { ade_neu }

operator /  (x : integer;
		    y : inte_type ) quot : inte_type;
begin
   quot:=intval(x)/y;
end;

global operator /  ( y : inte_type; x : real) quot : inte_type;
begin
   quot:=y/intval(x);
end; { ade_neu }

global operator /  ( y : inte_type; x : integer) quot : inte_type;
begin
   quot:=y/intval(x);
end; { ade_neu }



{==========   Standardfunktionen :     ======================}

global function sqr (x	: inte_type) : inte_type;
var xq	: interval;
   hilf	: integer; 
begin
   case type_switch of
      1:
	sqr.rint:= sqr(x.rint); 
      2: 
	 if (x.a) then
	 begin
	    if (expo(sup(x.rint))>511) then
	    begin
	       sqr.rint := 0;
	       sqr.a   := false;
	    end
	    else
	    begin
	       sqr.rint := sqr(x.rint); 
	       sqr.a   := true;
	    end
	 end
	 else
	 begin
	    sqr.rint := 0;
	    sqr.a   := false;
	 end; 

      3:
	if (rerror) then
	   sqr :=iconstr(intval(0), 0.0, 0.0)
	else
	begin
	   if (max(expo(sup(x.rint)), expo(inf(x.rint))) >510) then
	   begin
	      sqr :=iconstr(intval(0), 0.0, 0.0);
	      rerror := true;
	   end
	   else
	   begin
	      xq := sqr(x.rint); 
	      sqr.rint := xq;
	      hilf := max(expo(sup(re(x.cint))),expo(inf(re(x.cint)))); 
	      hilf := max(hilf, expo(im(x.cint))); 
	      if ((hilf <510) and x.a) then
	      begin
		 sqr.cint := sqr(x.cint); 
		 sqr.a    := true;
	      end
	      else
	      begin
		 sqr.cint := icompl(intval(0)); 
		 sqr.a    := false;
	      end;
	      if (x.d) then
	      begin
		 sqr.dint := 2*x.dint * x.rint;
		 sqr.d    := true
	      end
	      else
	      begin
		 sqr.dint :=0; 
		 sqr.d    := false;
	      end;
	   end;
	end;
   end; { sqr }
   end;


global function pow (  x: interval;  n:integer) : interval;
var xq	    : interval;
   hilf	    : real;
   k, i, si : integer;
   pot2	    : array[1..1030] of interval; 
   prod	    : interval; 
begin
   if (n<0) then
   begin
	 pow:=intval(1.0)/pow(x, -n)
   end
   else
   begin
      if (n=0) then
      begin
	 if (0 in x) then         {===== Ausdruck 0 hoch 0 ======}
	 begin
	    writeln(' Fehler :  Ausdruck 0 hoch 0 '); 
	    pow := intval(0); 
	 end
	 else
	 begin
	    pow := intval(1);
	 end
      end
      else
      begin                  { ===========  n>0 =====================}
	 k:=2;
	 i:=1;
	 pot2[1] := x;
	 while (k<=n) do
	 begin
	    i:=i+1; 
	    pot2[i]:=sqr(pot2[i-1]);
	    k:=2*k;
	 end;
      end;
      k:=k div 2;
      prod:=pot2[i];
      si:=k; 
      while (si<n) do
      begin
	 k:=k div 2;
	 i:=i-1; 
	 if (si+k<=n) then
	 begin
	    prod:=prod*pot2[i];
	    si:=si+k;
	 end;
      end;
      pow:=prod; 
   end;
end; { pow }
   

global function pow(x : inte_type;
		    n : integer) : inte_type; 
var xq	    : interval;
   hilf	    : real;
   k, i, si : integer;
   cpot2    : array[1..1030] of icom; 
   pot2	    : array[1..1030] of interval; 
   cprod	    : icom; 
   prod	    : interval; 
begin
   case type_switch of 
     1:
       if (n<0) then
       begin
	  pow.rint:=intval(1.0)/pow(x.rint, -n)
       end
       else
       begin
	  if (n=0) then
	  begin
	     if (0 in x.rint) then         {===== Ausdruck 0 hoch 0 ======}
	     begin
		writeln(' Fehler :  Ausdruck 0 hoch 0 '); 
		pow.rint := intval(0); 
	     end
	     else
	     begin
		pow.rint := intval(1);
	     end
	  end
	  else
	  begin                  { ===========  n>0 =====================}
	     k:=2;
	     i:=1;
	     pot2[1] := x.rint;
	     while (k<=n) do
	     begin
		i:=i+1; 
		pot2[i]:=sqr(pot2[i-1]);
		k:=2*k;
	     end;
	  end;
	  k:=k div 2;
	  prod:=pot2[i];
	  si:=k; 
	  while (si<n) do
	  begin
	     k:=k div 2;
	     i:=i-1; 
	     if (si+k<=n) then
	     begin
		prod:=prod*pot2[i];
	    si:=si+k;
	     end;
	  end;
	  pow.rint:=prod; 
       end;
  
      2:
	if (not(x.a)) then
	    begin
	       pow.a := false;
	       pow.rint := intval(0);
	    end
	    else
	       if (n<0) then
		  pow:=intval(1.0)/pow(x, -n)
	       else
	       begin
		  if (n=0) then
		  begin
		     if (0 in x.rint) then    {===== Ausdruck 0 hoch 0 ======}
		     begin
			pow.a    := false; 
			pow.rint := intval(0); 
		     end
		     else
		     begin
			pow.rint := intval(1);
			pow.a    := true; 
		     end
		  end
		  else
		  begin       { ===========  n>0 =====================}
		     if (n*max(expo(sup(x.rint)), expo(inf(x.rint))) >1020) then
		     begin
			pow.rint := intval(0) ;
			pow.a   := false;
		     end
		     else
		     begin
			pow.a := true; 
			k:=2;
			i:=1;
			pot2[1] := x.rint;
			while (k<=n) do
			begin
			   i:=i+1; 
			   pot2[i]:=sqr(pot2[i-1]);
			   k:=2*k;
			end;
		     end;
		     k:=k div 2;
		     prod:=pot2[i];
		     si:=k; 
		     while (si<n) do
		     begin
			k:=k div 2;
			i:=i-1; 
			if (si+k<=n) then
			begin
			   prod:=prod*pot2[i];
			   si:=si+k;
			end;
		     end;
		     pow.rint:=prod; 
		  end;
	       end;
  
 

      3:
	if (rerror) then
	   pow :=iconstr(intval(0), 0.0, 0.0)
	else
	   if (n<0) then
	      pow:=intval(1.0)/pow(x, -n)
	   else
	   begin
	      if (n=0) then
		 if (0 in x.rint) then
		 begin
		    rerror := true; 
		    pow :=iconstr(intval(0), 0.0, 0.0)
		 end
		 else
		 begin
		    pow.rint := intval(1);
		    pow.dint := intval(0);
		    pow.d    := true; 
		    if (x.a) then
		    begin
		       if (0 in x.cint.re) then
		       begin
			  pow.cint := icompl(intval(0)); 
			  pow.a := false;
		       end		  
		       else
		       begin
			  pow.cint := icompl(intval(1)); 
			  pow.a := true;
		       end;
		    end;
		 end
		 else
		 begin
		    if (n*max(expo(sup(x.rint)), expo(inf(x.rint))) >1020) then
		    begin
		       pow :=iconstr(intval(0), 0.0, 0.0);
		       rerror := true;
		    end
		    else
		    begin
		       if (n=1) then  pow := x;
		       if (n=2) then pow:=sqr(x);
		       if (n>=3) then
		       begin
			  pow.rint := pow(x.rint, n);
			  if (x.d) then
			  begin
			     pow.dint := n*x.dint*pow(x.rint, n-1);
			     pow.d := true
			  end
			  else
			  begin
			     pow.dint:=intval(0);
			     pow.d :=false;
			  end; 
			  if (x.a and (n*expo(sup(abs(x.cint))) <1000)) then
			  begin   
			     k:=2;
			     i:=1;
			     cpot2[1] := x.cint;
			     while (k<=n) do
			     begin
				i:=i+1; 
				cpot2[i]:=sqr(cpot2[i-1]);
				k:=2*k;
			     end;
			     k:=k div 2;
			     cprod:=cpot2[i];
			     si:=k; 
			     while (si<n) do
			     begin
				k:=k div 2;
				i:=i-1; 
				if (si+k<=n) then
				begin
				   cprod:=cprod*cpot2[i];
				   si:=si+k;
				end;
			     end;
			     pow.cint:=cprod;
			     pow.a:=true; 
			  end
			  else
			  begin
			     pow.cint := icompl(intval(0)); 
			     pow.a := false;
			  end;		  
		       end; 
		    end;
		 end;
	   end;
   end; { pow }
   end;
 

global function pospart ( x : inte_type) : inte_type;
begin
   case type_switch of 
     2:
       if (x.a) then
       begin
	  if (0>=sup(x.rint)) then
	  begin
	     pospart.rint := 0;
	     pospart.a   := true;
	  end
	  else
	  begin
	     if (0 <= inf(x.rint)) then
		pospart:=x
	     else
	     begin
		pospart.rint := 0.0 +* x.rint;
		pospart.a   := true;
	     end
	  end
       end
       else
       begin
	  pospart.rint := 0;
	  pospart.a   := false;
       end;
      
3:
       if (rerror) then
	  pospart :=iconstr(intval(0), 0.0, 0.0)
       else
       begin
	  if (0 >= sup(x.rint)) then
	     pospart :=iconstr(intval(0), 0.0, 0.0)
	  else
	  begin
	     if (0 <= inf(x.rint)) then
		pospart:=x
	     else
	     begin
		pospart.rint :=intval(0.0, sup(x.rint)); 
		pospart.dint := 0.0 +* x.dint;
		pospart.d    := x.d;
		pospart.cint := icompl(intval(0)); 
		pospart.a    := false;
	     end;
	  end;
       end;
   end;
   end; { pospart }


global function exp (x	: inte_type) : inte_type;
var ehx	: interval; 
begin
   case type_switch of	    
      1:		    
	    exp.rint:= exp(x.rint);
      2:		    
	    if (x.a) then
	    begin
	       if (sup(x.rint)>709) then
	       begin
		  exp.rint := 0;
		  exp.a   := false;
	       end
	       else
	       begin
		  exp.rint := exp(x.rint); 
		  exp.a   := true;
	       end
	    end
	    else
		  begin
		     exp.rint := 0;
		     exp.a   := false;
		  end;
      
3:

  
  if (rerror) then
     exp :=iconstr(intval(0), 0.0, 0.0)
  else
  begin
     if (sup(x.rint)>709) then
     begin
	exp :=iconstr(intval(0), 0.0, 0.0);
	rerror := true;
      end
      else
      begin
	 ehx := exp(x.rint); 
	 exp.rint := ehx; 
	 if ((sup(re(x.cint))<=709) and x.a) then
	 begin
	    exp.cint := exp(x.cint); 
	    exp.a    := true;
	 end
	 else
	 begin
	    exp.cint := icompl(intval(0)); 
	    exp.a    := false;
	 end;
	 if (x.d) then
	 begin
	    exp.dint := x.dint * ehx;
	    exp.d    := true
	 end
	 else
	 begin
	    exp.dint :=0; 
	    exp.d    := false;
	 end;
      end;
  end;
end; { exp }
   end;

global function sin (x	: inte_type) : inte_type;
var sx, cx : interval; 
begin
   case type_switch of	    
      1:		    
	    sin.rint:= sin(x.rint);
      2:
	if (x.a) then
	begin
	   sin.rint := sin(x.rint); 
	   sin.a   := true;
	end
	else
	begin
	   sin.rint := intval(-1,1);
	   sin.a   := true;
	end;
3:    
      if (rerror) then
	 sin :=iconstr(intval(0), 0.0, 0.0)
      else
      begin
	 sx := sin(x.rint); 
	 cx := cos(x.rint); 
	 sin.rint := sx; 
	 if ((x.cint.im<=709) and x.a) then
	 begin
	    sin.cint := sin(x.cint); 
	    sin.a    := true;
      end
      else
      begin
	 sin.cint := icompl(intval(0)); 
	 sin.a    := false;
      end;
      if (x.d) then
      begin
	 sin.dint := x.dint * cx;
	 sin.d    := true
      end
      else
      begin
	 sin.dint :=0; 
	 sin.d    := false;
      end;
      end;
   end; { sin }
   end;

global function cos (x	: inte_type) : inte_type;
var sx, cx : interval; 
begin
   case type_switch of	    
      1:		    
	    cos.rint:= cos(x.rint);
      2:
	if (x.a) then
	begin
	   cos.rint := cos(x.rint); 
	   cos.a   := true;
	end
	else
	begin
	   cos.rint := intval(-1,1);
	   cos.a   := true;
	end;
      3:
	if (rerror) then
	   cos :=iconstr(intval(0), 0.0, 0.0)
	else
	begin
      sx := sin(x.rint); 
	   cx := cos(x.rint); 
      cos.rint := cx; 
	   if ((x.cint.im<=709) and x.a) then
	   begin
	 cos.cint := cos(x.cint); 
	      cos.a    := true;
	   end
	   else
	   begin
	      cos.cint := icompl(intval(0)); 
	      cos.a    := false;
	   end;
	   if (x.d) then
	   begin
	      cos.dint := -x.dint * sx;
	      cos.d    := true
	   end
	   else
	   begin
	      cos.dint :=0; 
	      cos.d    := false;
	   end;
	end;
   end;
end; { cos }


global function sqrtabs (x : inte_type) : inte_type;
var   wurzel : interval; 
begin
   
   case type_switch of	    
      1:		    
	    sqrtabs.rint:= sqrt(abs(x.rint));
      2: 
   if (x.a) then
   begin
      sqrtabs.rint := sqrt(abs(x.rint)); 
      sqrtabs.a   := true;
   end
   else
   begin
      sqrtabs.rint := 0;
      sqrtabs.a   := false;
   end;

	    
3:    
   if (rerror) then
      sqrtabs :=iconstr(intval(0), 0.0, 0.0)
   else
   begin
      wurzel := sqrt(abs(x.rint));
      sqrtabs.rint := wurzel; 
      if (x.a or x.d) then
      begin
	 if (sup(x.rint)<0.0) then
	 begin
	    if (x.d) then
	    begin
	       sqrtabs.dint := -x.dint/(2*wurzel);
	       sqrtabs.d    := true;
	    end
	    else
	    begin
	       sqrtabs.dint := intval(0); 
	       sqrtabs.d    := false;
	    end; 
	    if (not(x.a)) then
	    begin
	       sqrtabs.cint := icompl(intval(0));
	       sqrtabs.a    := false;
	    end
	    else
	    begin
	       if (sup(re(x.cint))<0.0) then
	       begin
		  sqrtabs.cint := sqrt(-x.cint); 
		  sqrtabs.a    := true
	       end
	       else
	       begin
		  sqrtabs.cint := icompl(intval(0));
		  sqrtabs.a    := false;
	       end
	    end
	 end
	 else
	 begin
	    if (inf(x.rint)>0.0) then
	    begin
	       if (x.d) then
	       begin
		  sqrtabs.dint := x.dint/(2*wurzel);
		  sqrtabs.d    := true;
	       end
	       else
	       begin
		  sqrtabs.dint := intval(0); 
		  sqrtabs.d    := false;
	       end; 
	       if (not(x.a)) then
	       begin
		  sqrtabs.cint := icompl(intval(0));
		  sqrtabs.a    := false;
	       end
	       else
	       begin
		  if (inf(re(x.cint))>0.0) then
		  begin
		     sqrtabs.cint := sqrt(x.cint); 
		     sqrtabs.a    := true
		  end
		  else
		  begin
		     sqrtabs.cint := icompl(intval(0));
		     sqrtabs.a    := false;
		  end
	       end
	    end
	    else {0 liegt in jedem Argumentbereich}
	    begin
	       sqrtabs.dint := intval(0); 
	       sqrtabs.d    := false;
	       sqrtabs.cint := icompl(intval(0));
	       sqrtabs.a    := false
	    end
	 end
      end
   end;
   end;
   end; { sqrtabs }
   

global function abs (x	: inte_type) : inte_type;
var abx	: interval; 
begin
   case type_switch of	    
      1:		    
	   abs.rint:= abs(x.rint);
      2:
	begin
	   abs.rint := abs(x.rint);
	   abs.a := x.a;
	end;
3:    




      if (rerror) then
	 abs :=iconstr(intval(0), 0.0, 0.0)
      else
      begin
	 abx := abs(x.rint); 
	 abs.rint := abx;
	 if (x.a or x.d) then
	 begin
	    if (sup(x.rint)<=0.0) then
	    begin
	       if (x.a) then
	       begin
		  abs.a:=true;
		  abs.cint := - x.cint
	       end
	       else
	       begin
	       abs.cint := icompl(intval(0)); 
		  abs.a    := false;
	       end;
	       if (x.d) then
	       begin
	       abs.d:=true;
		  abs.dint := - x.dint
	       end
	       else
	    begin
	       abs.dint := intval(0); 
	       abs.d    := false;
	    end;
	    end
	    else
	    begin
	       if (inf(x.rint)>=0.0) then
	    begin
	       if (x.a) then
	       begin
		  abs.a:=true;
		  abs.cint :=   x.cint
	       end
	       else
	       begin
		  abs.cint := icompl(intval(0)); 
		  abs.a    := false;
	       end;
	       if (x.d) then
	       begin
		  abs.d:=true;
		  abs.dint :=   x.dint
	       end
	       else
	       begin
		  abs.dint := intval(0); 
		  abs.d    := false;
	       end;
	    end
	    else
	    begin
	       abs.a := false;
	       if (x.d) then
	       begin
		  abs.d := true;
		  abs.dint := (-x.dint)+* x.dint 
	       end
	       else
	       begin
		  abs.dint := intval(0); 
		  abs.d    := false;
	       end
	    end;
	 end;
	 end
	 else
	 begin 
	 abs.cint := icompl(intval(0)); 
	    abs.a    := false;
	    abs.dint := intval(0); 
	 abs.d    := false;
	 end;
      end;
   end;
end; { abs }
   
   


end.

